﻿using System;
using System.Threading.Tasks;

namespace InstanceFactory.CustomPrivateAccessor.ConsoleApplication._4._5
{
  /// <summary>
  /// Implementation of a class with non-public members and .NET 4.5 features.
  /// </summary>
  public class ClassWithNonPublicMembers
  {
    #region Public Properties

    /// <summary>
    /// Gets / sets the value of a public property.
    /// </summary>
    public string PublicProperty { get; set; }

    #endregion Public Properties


    #region Protected Properties

    /// <summary>
    /// Gets / sets the value of a protected property.
    /// </summary>
    protected string ProtectedProperty { get; set; }

    #endregion Protected Properties


    #region Private Properties

    /// <summary>
    /// Gets / sets the value of a private static property.
    /// </summary>
    private string PrivateProperty { get; set; }

    #endregion Private Properties


    #region Private Static Properties

    /// <summary>
    /// Gets / sets the value of a private property.
    /// </summary>
    private static string PrivateStaticProperty { get; set; }

    #endregion Private Static Properties


    #region Private Methods

    /// <summary>
    /// A private async method with two input parameters and a return value.
    /// </summary>
    /// <param name="millisecondsDelay">Delaytime in milliseconds..</param>
    /// <param name="outputText">Output text.</param>
    /// <returns>Delaytime in seconds.</returns>
    private async Task<double> PrivateMethodWithReturnValueAsync
      (
      int millisecondsDelay,
      string outputText
      )
    {
      // First wait.
      await Task.Delay(millisecondsDelay);

      // Write the output
      Console.WriteLine(">{0}< milliseconds delayed. Output: >{1}<", millisecondsDelay, outputText);

      // Need to cast to make sure the all values will be handled as double, not as int.
      return ((double)millisecondsDelay / (double)1000);
    }

    #endregion Private Methods


    #region Private Static Methods

    /// <summary>
    /// A private static async method with two input parameters and no return value.
    /// </summary>
    /// <param name="millisecondsDelay">Delaytime in milliseconds..</param>
    /// <param name="outputText">Output text.</param>
    private static async Task PrivateStaticVoidMethodAsync
      (
      int millisecondsDelay,
      string outputText
      )
    {
      // First wait.
      await Task.Delay(millisecondsDelay);

      // Do something that can be unit tested
      ClassWithNonPublicMembers.PrivateStaticProperty = outputText;

      // Write the output
      Console.WriteLine(">{0}< milliseconds delayed. Output: >{1}<", millisecondsDelay, outputText);
    }

    #endregion Private Static Methods


    #region Protected Methods

    /// <summary>
    /// A simple protected method with a parameter.
    /// </summary>
    /// <param name="parameter">The parameter</param>
    /// <returns>Another string.</returns>
    protected string ProtectedMethod
      (
      string parameter
      )
    {
      Console.WriteLine("Parameter: >{0}<", parameter);

      return (parameter + " processed");
    }

    #endregion Protected Methods
  }
}
