﻿using InstanceFactory.CustomPrivateAccessor.ConsoleApplication;
using InstanceFactory.CustomPrivateAccessor.ConsoleApplication.Test.TestTools.Accessors;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Diagnostics;

namespace InstanceFactory.CustomPrivateAccessor.ConsoleApplication.Test
{
  /// <summary>
  ///This is a test class for ClassWithNonPublicMemberTest and is intended
  ///to contain all ClassWithNonPublicMemberTest Unit Tests
  ///</summary>
  [TestClass()]
  public class ClassWithNonPublicMemberTest
  {
    private TestContext testContextInstance;

    /// <summary>
    ///Gets or sets the test context which provides
    ///information about and functionality for the current test run.
    ///</summary>
    public TestContext TestContext
    {
      get
      {
        return testContextInstance;
      }
      set
      {
        testContextInstance = value;
      }
    }

    #region Additional test attributes
    // 
    //You can use the following additional attributes as you write your tests:
    //
    //Use ClassInitialize to run code before running the first test in the class
    //[ClassInitialize()]
    //public static void MyClassInitialize(TestContext testContext)
    //{
    //}
    //
    //Use ClassCleanup to run code after all tests in a class have run
    //[ClassCleanup()]
    //public static void MyClassCleanup()
    //{
    //}
    //
    //Use TestInitialize to run code before running each test
    //[TestInitialize()]
    //public void MyTestInitialize()
    //{
    //}
    //
    //Use TestCleanup to run code after each test has run
    //[TestCleanup()]
    //public void MyTestCleanup()
    //{
    //}
    //
    #endregion


    #region Test Methods

    /// <summary>
    /// Shows how to access / test a protected property.
    /// </summary>
    [TestMethod]
    public void TestProtectedProperty()
    {
      // Create the accessor
      ClassWithNonPublicMembersAccessor accessor = new ClassWithNonPublicMembersAccessor();

      string testValue = "static unit test";

      // Set a value
      accessor.ProtectedProperty = testValue;

      // Write it out for tracing ablilities
      Debug.WriteLine("ProtectedProperty: >{0}<{1}", accessor.ProtectedProperty, Environment.NewLine);

      // Test the value (means call the getter)
      Assert.AreEqual(testValue, accessor.ProtectedProperty, "Property has wrong value");
    }

    /// <summary>
    /// Shows how to access / test a private property.
    /// </summary>
    [TestMethod]
    public void TestPrivateProperty()
    {
      // Create the accessor
      ClassWithNonPublicMembersAccessor accessor = new ClassWithNonPublicMembersAccessor();

      string testValue = "static unit test";

      // Set a value
      accessor.PrivateProperty = testValue;

      // Write it out for tracing ablilities
      Debug.WriteLine("PrivateProperty: >{0}<{1}", accessor.PrivateProperty, Environment.NewLine);

      // Test the value (means call the getter)
      Assert.AreEqual(testValue, accessor.PrivateProperty, "Property has wrong value");
    }

    /// <summary>
    /// Shows how to access / test a private static property.
    /// </summary>
    [TestMethod]
    public void TestPrivateStaticProperty()
    {
      string testValue = "static unit test";

      // Set a value
      ClassWithNonPublicMembersAccessor.PrivateStaticProperty = testValue;

      // Write it out for tracing ablilities
      Debug.WriteLine("PrivateProperty: >{0}<{1}", ClassWithNonPublicMembersAccessor.PrivateStaticProperty, Environment.NewLine);

      // Test the value (means call the getter)
      Assert.AreEqual(testValue, ClassWithNonPublicMembersAccessor.PrivateStaticProperty, "Property has wrong value");
    }

    /// <summary>
    /// Shows how to call a protected method with parameter and return value.
    /// </summary>
    [TestMethod]
    public void TestProtectedMethod()
    {
      string testValue = "method unit test";

      // Create the accessor
      ClassWithNonPublicMembersAccessor accessor = new ClassWithNonPublicMembersAccessor();

      string result = accessor.ProtectedMethod(testValue);

      // Write it out for tracing ablilities
      Debug.WriteLine("Result: >{0}<{1}", result, Environment.NewLine);

      // Do some testing
      Assert.AreEqual(testValue + " processed", result, "Result has wrong value");
    }

    #endregion Test Methods
  }
}
